@@ -31,7 +31,7 @@ helper_parse(int c, char **argv, int invert, unsigned int *flags,
 		if (*flags)
 			xtables_error(PARAMETER_PROBLEM,
 					"helper match: Only use --helper ONCE!");
-		xtables_check_inverse(optarg, &invert, &invert, 0);
+		xtables_check_inverse(optarg, &invert, &optind, 0, argv);
 		strncpy(info->name, optarg, 29);
 		info->name[29] = '\0';
 		if (invert)
@@ -69,20 +69,7 @@ static void helper_save(const void *ip, const struct xt_entry_match *match)
 }
 
 static struct xtables_match helper_match = {
-	.family		= NFPROTO_IPV4,
-	.name		= "helper",
-	.version	= XTABLES_VERSION,
-	.size		= XT_ALIGN(sizeof(struct xt_helper_info)),
-	.help		= helper_help,
-	.parse		= helper_parse,
-	.final_check	= helper_check,
-	.print		= helper_print,
-	.save		= helper_save,
-	.extra_opts	= helper_opts,
-};
-
-static struct xtables_match helper_match6 = {
-	.family		= NFPROTO_IPV6,
+	.family		= NFPROTO_UNSPEC,
 	.name		= "helper",
 	.version	= XTABLES_VERSION,
 	.size		= XT_ALIGN(sizeof(struct xt_helper_info)),
@@ -97,5 +84,4 @@ static struct xtables_match helper_match6 = {
 void _init(void)
 {
 	xtables_register_match(&helper_match);
-	xtables_register_match(&helper_match6);
 }
